# Copyright 2022, Digi International Inc.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

class ConnectCoreBLEException(Exception):
    """
    Generic ConnectCore BLE exception. This class and its subclasses indicate
    conditions that an application might want to catch when an error occurs.

    All functionality of this class is the inherited of `Exception
    <https://docs.python.org/2/library/exceptions.html?highlight=exceptions.exception#exceptions.Exception>`_.
    """


class BluetoothNotSupportedException(ConnectCoreBLEException):
    """
    This exception is thrown when Bluetooth is not supported by the device in any form,
    either native support or through an XBee device.

    All functionality of this class is the inherited of `Exception
    <https://docs.python.org/2/library/exceptions.html?highlight=exceptions.exception#exceptions.Exception>`_.
    """

    # Constants.
    __DEFAULT_MESSAGE = "Bluetooth is not supported in the system, neither natively nor through an XBee device."

    def __init__(self, message=__DEFAULT_MESSAGE):
        ConnectCoreBLEException.__init__(self, message)


class NotAuthenticatedException(ConnectCoreBLEException):
    """
    This exception is thrown when the client attempts to communicate before a successful
    authentication process.

    All functionality of this class is the inherited of `Exception
    <https://docs.python.org/2/library/exceptions.html?highlight=exceptions.exception#exceptions.Exception>`_.
    """

    # Constants.
    __DEFAULT_MESSAGE = "Data cannot be encrypted/decrypted, user has not been authenticated"

    def __init__(self, message=__DEFAULT_MESSAGE):
        ConnectCoreBLEException.__init__(self, message)



# Copyright 2022, Digi International Inc.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

def hex_to_string(byte_array, pretty=True):
    """
    Returns the provided bytearray in string format.

    Args:
        byte_array (Bytearray): the bytearray to format as string.
        pretty (Boolean, optional): `True` to use a white space separator between bytes,
                                    `False` otherwise.

    Returns:
        String: the bytearray formatted as string.
    """
    separator = " " if pretty else ""
    return separator.join(["%02X" % i for i in byte_array])


def doc_enum(enum_class, descriptions=None):
    """
    Returns a string with the description of each value of an enumeration.

    Args:
        enum_class (Enumeration): the Enumeration to get its values documentation.
        descriptions (dictionary): each enumeration's item description. The key is the enumeration
                                   element name and the value is the description.

    Returns:
        String: the string listing all the enumeration values and their descriptions.
    """
    tab = " "*4
    data = "\n| Values:\n"
    for item in enum_class:
        data += """| {:s}**{:s}**{:s} {:s}\n""".format(
            tab, str(item), ":" if descriptions is not None else " =",
            str(item.value) if descriptions is None else descriptions[item])
    return data + "| \n"



# Copyright 2022, Digi International Inc.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import sys
import threading

from digi.ccble.exceptions import BluetoothNotSupportedException, ConnectCoreBLEException
from digi.ccble.service import ConnectCoreBLEService

# Variables.
cc_ble_service = None


def _data_received_callback(data):
    """
    Callback to be notified when new data is received from the connected device.

    Args:
        data (Bytearray): received (JSON) data.
    """
    # Send data back.
    try:
        cc_ble_service.send_data(data)
    except ConnectCoreBLEException as exc:
        print(f"Error sending data: {str(exc)}")


def _connection_callback():
    """
    Callback to be notified when a new connection is established.
    """
    print(f"Connection established through '{cc_ble_service.get_interface_type().title}'")


def main():
    global cc_ble_service

    # Initialize service.
    try:
        cc_ble_service = ConnectCoreBLEService.get_instance()
    except BluetoothNotSupportedException:
        print("The system does not support bluetooth. Aborting...")
        sys.exit(1)
    # Register service callbacks.
    cc_ble_service.add_data_received_callback(_data_received_callback)
    cc_ble_service.add_connect_callback(_connection_callback)
    # Configure thread for service execution.
    application_thread = threading.Thread(target=cc_ble_service.start, daemon=True)
    application_thread.start()
    # Wait for input to stop application.
    print('Service started, hit <ENTER> to stop and exit.')
    try:
        input('> ')
        # Stop the service.
        print('Stopping the service...')
        try:
            cc_ble_service.stop()
            application_thread.join()
            application_thread = threading.Thread(target=cc_ble_service.start, daemon=True)
            print('Service stopped!')
        except ConnectCoreBLEException as exc:
            print(str(exc))
    except KeyboardInterrupt:
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright 2022-2024, Digi International Inc.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

__version__ = '1.0.7'
__title__ = 'digi-connectcore-ble'
__description__ = 'Digi ConnectCore Bluetooth Low Energy library'
__url__ = 'https://github.com/digi-embedded/connectcore-ble-python'
__author__ = 'Digi International Inc.'
__author_email__ = 'tech.support@digi.com'
__license__ = 'Mozilla Public License 2.0 (MPL 2.0)'
__copyright__ = '2022, 2023, Digi International Inc.'
